@@ -5,7 +5,7 @@
 /*	Postfix alias database maintenance
 /* SYNOPSIS
 /* .fi
-/*	\fBpostalias\fR [\fB-Nfinoprsuvw\fR] [\fB-c \fIconfig_dir\fR]
+/*	\fBpostalias\fR [\fB-Nfijnoprsuvw\fR] [\fB-c \fIconfig_dir\fR]
 /*	[\fB-d \fIkey\fR] [\fB-q \fIkey\fR]
 /*		[\fIfile_type\fR:]\fIfile_name\fR ...
 /* DESCRIPTION
@@ -56,6 +56,11 @@
 /*	Incremental mode. Read entries from standard input and do not
 /*	truncate an existing database. By default, \fBpostalias\fR(1) creates
 /*	a new database from the entries in \fIfile_name\fR.
+/* .IP \fB-j\fR
+/*	JSON output. Format the output from \fB-q\fR and \fB-s\fR
+/*	as one \fB{"\fIkey\fB": "\fIvalue\fB"}\fR object per line.
+/* .sp
+/*	This feature is available in Postfix version 3.11 and later.
 /* .IP \fB-N\fR
 /*	Include the terminating null character that terminates lookup keys
 /*	and values. By default, \fBpostalias\fR(1) does whatever
@@ -113,6 +118,11 @@
 /*	The database type. To find out what types are supported, use
 /*	the "\fBpostconf -m\fR" command.
 /*
+/*	When no \fIfile_type\fR is specified, the software uses the database
+/*	type specified via the \fBdefault_database_type\fR configuration
+/*	parameter.
+/*	The default value for this parameter depends on the host environment.
+/*
 /*	The \fBpostalias\fR(1) command can query any supported file type,
 /*	but it can create only the following file types:
 /* .RS
@@ -142,11 +152,6 @@
 /*	The output consists of two files, named \fIfile_name\fB.pag\fR and
 /*	\fIfile_name\fB.dir\fR.
 /*	This is available on systems with support for \fBsdbm\fR databases.
-/* .PP
-/*	When no \fIfile_type\fR is specified, the software uses the database
-/*	type specified via the \fBdefault_database_type\fR configuration
-/*	parameter.
-/*	The default value for this parameter depends on the host environment.
 /* .RE
 /* .IP \fIfile_name\fR
 /*	The name of the alias database source file when creating a database.
@@ -249,6 +254,7 @@
 #include <fcntl.h>
 #include <ctype.h>
 #include <string.h>
+#include <inttypes.h>
 
 /* Utility library. */
 
@@ -287,6 +293,13 @@
 #define POSTALIAS_FLAG_SAVE_PERM	(1<<1)	/* copy access permission
 						 * from source */
 
+ /*
+  * Global state.
+  */
+int     json_output;
+VSTRING *json_key_buf;
+VSTRING *json_val_buf;
+
 /* postalias - create or update alias database */
 
 static void postalias(char *map_type, char *path_name, int postalias_flags,
@@ -487,10 +500,10 @@ static void postalias(char *map_type, char *path_name, int postalias_flags,
      * this information MUST be written without a trailing null appended to
      * key or value.
      */
+#if (defined(HAS_NIS) || defined(HAS_NISPLUS))
     mkmap->dict->flags &= ~DICT_FLAG_TRY1NULL;
     mkmap->dict->flags |= DICT_FLAG_TRY0NULL;
-    vstring_sprintf(value_buffer, "%010ld", (long) time((time_t *) 0));
-#if (defined(HAS_NIS) || defined(HAS_NISPLUS))
+    vstring_sprintf(value_buffer, "%010" PRId64, (int64_t) time((time_t *) 0));
     mkmap->dict->flags &= ~DICT_FLAG_FOLD_FIX;
     mkmap_append(mkmap, "YP_LAST_MODIFIED", STR(value_buffer));
     mkmap_append(mkmap, "YP_MASTER_NAME", var_myhostname);
@@ -553,7 +566,12 @@ static int postalias_queries(VSTREAM *in, char **maps, const int map_count,
 		    msg_warn("table %s:%s should return NO RESULT in case of NOT FOUND",
 			     dicts[n]->type, dicts[n]->name);
 		}
-		vstream_printf("%s:	%s\n", STR(keybuf), value);
+		if (json_output == 0)
+		    vstream_printf("%s:	%s\n", STR(keybuf), value);
+		else
+		    vstream_printf("{\"%s\": \"%s\"}\n",
+			      quote_for_json(json_key_buf, STR(keybuf), -1),
+				   quote_for_json(json_val_buf, value, -1));
 		found = 1;
 		break;
 	    }
@@ -593,7 +611,12 @@ static int postalias_query(const char *map_type, const char *map_name,
 	    msg_warn("table %s:%s should return NO RESULT in case of NOT FOUND",
 		     map_type, map_name);
 	}
-	vstream_printf("%s\n", value);
+	if (json_output == 0)
+	    vstream_printf("%s\n", value);
+	else
+	    vstream_printf("{\"%s\": \"%s\"}\n",
+			   quote_for_json(json_key_buf, key, -1),
+			   quote_for_json(json_val_buf, value, -1));
     }
     if (dict->error)
 	msg_fatal("table %s:%s: query error: %m", dict->type, dict->name);
@@ -703,7 +726,12 @@ static void postalias_seq(const char *map_type, const char *map_name,
 	    msg_warn("table %s:%s should return NO RESULT in case of NOT FOUND",
 		     map_type, map_name);
 	}
-	vstream_printf("%s:	%s\n", key, value);
+	if (json_output == 0)
+	    vstream_printf("%s:	%s\n", key, value);
+	else
+	    vstream_printf("{\"%s\": \"%s\"}\n",
+			   quote_for_json(json_key_buf, key, -1),
+			   quote_for_json(json_val_buf, value, -1));
     }
     if (dict->error)
 	msg_fatal("table %s:%s: sequence error: %m", dict->type, dict->name);
@@ -783,7 +811,7 @@ int     main(int argc, char **argv)
     /*
      * Parse JCL.
      */
-    while ((ch = GETOPT(argc, argv, "Nc:d:finopq:rsuvw")) > 0) {
+    while ((ch = GETOPT(argc, argv, "Nc:d:fijnopq:rsuvw")) > 0) {
 	switch (ch) {
 	default:
 	    usage(argv[0]);
@@ -810,6 +838,13 @@ int     main(int argc, char **argv)
 	    update = 1;
 	    open_flags &= ~O_TRUNC;
 	    break;
+	case 'j':
+	    if (json_output == 0) {
+		json_output = 1;
+		json_key_buf = vstring_alloc(100);
+		json_val_buf = vstring_alloc(100);
+	    }
+	    break;
 	case 'n':
 	    dict_flags |= DICT_FLAG_TRY0NULL;
 	    dict_flags &= ~DICT_FLAG_TRY1NULL;
@@ -846,6 +881,8 @@ int     main(int argc, char **argv)
 	    break;
 	}
     }
+    if (json_output && !(sequence || query))
+	msg_fatal("option -j requires -q or -s");
     mail_conf_read();
     /* Enforce consistent operation of different Postfix parts. */
     import_env = mail_parm_split(VAR_IMPORT_ENVIRON, var_import_environ);
