@@ -5,7 +5,7 @@
 /*	Postfix lookup table management
 /* SYNOPSIS
 /* .fi
-/*	\fBpostmap\fR [\fB-bfFhimnNoprsuUvw\fR] [\fB-c \fIconfig_dir\fR]
+/*	\fBpostmap\fR [\fB-bfFhijmnNoprsuUvw\fR] [\fB-c \fIconfig_dir\fR]
 /*	[\fB-d \fIkey\fR] [\fB-q \fIkey\fR]
 /*		[\fIfile_type\fR:]\fIfile_name\fR ...
 /* DESCRIPTION
@@ -136,6 +136,11 @@
 /*	Incremental mode. Read entries from standard input and do not
 /*	truncate an existing database. By default, \fBpostmap\fR(1) creates
 /*	a new database from the entries in \fBfile_name\fR.
+/* .IP \fB-j\fR
+/*	JSON output. Format the output from \fB-q\fR and \fB-s\fR
+/*	as one \fB{"\fIkey\fB": "\fIvalue\fB"}\fR object per line.
+/* .sp
+/*	This feature is available in Postfix version 3.11 and later.
 /* .IP \fB-m\fR
 /*	Enable MIME parsing with "\fB-b\fR" and "\fB-h\fR".
 /* .sp
@@ -202,6 +207,11 @@
 /*	The database type. To find out what types are supported, use
 /*	the "\fBpostconf -m\fR" command.
 /*
+/*	When no \fIfile_type\fR is specified, the software uses the database
+/*	type specified via the \fBdefault_database_type\fR configuration
+/*	parameter.
+/*	The default value for this parameter depends on the host environment.
+/*
 /*	The \fBpostmap\fR(1) command can query any supported file type,
 /*	but it can create only the following file types:
 /* .RS
@@ -231,10 +241,6 @@
 /*	The output consists of two files, named \fIfile_name\fB.pag\fR and
 /*	\fIfile_name\fB.dir\fR.
 /*	This is available on systems with support for \fBsdbm\fR databases.
-/* .PP
-/*	When no \fIfile_type\fR is specified, the software uses the database
-/*	type specified via the \fBdefault_database_type\fR configuration
-/*	parameter.
 /* .RE
 /* .IP \fIfile_name\fR
 /*	The name of the lookup table source file when rebuilding a database.
@@ -386,6 +392,13 @@ typedef struct {
     int     found;			/* result */
 } POSTMAP_KEY_STATE;
 
+ /*
+  * Global state.
+  */
+int     json_output;
+VSTRING *json_key_buf;
+VSTRING *json_val_buf;
+
 /* postmap - create or update mapping database */
 
 static void postmap(char *map_type, char *path_name, int postmap_flags,
@@ -615,7 +628,13 @@ static void postmap_body(void *ptr, int unused_rec_type,
 		msg_warn("table %s:%s should return NO RESULT in case of NOT FOUND",
 			 dicts[n]->type, dicts[n]->name);
 	    }
-	    vstream_printf("%s	%s\n", keybuf, value);
+	    if (json_output == 0)
+		vstream_printf("%s	%s\n", keybuf, value);
+	    else
+		vstream_printf("{\"%s\": \"%s\"}\n",
+			       quote_for_json(json_key_buf, keybuf, -1),
+			       quote_for_json(json_val_buf, value, -1));
+
 	    state->found = 1;
 	    break;
 	}
@@ -698,7 +717,12 @@ static int postmap_queries(VSTREAM *in, char **maps, const int map_count,
 			msg_warn("table %s:%s should return NO RESULT in case of NOT FOUND",
 				 dicts[n]->type, dicts[n]->name);
 		    }
-		    vstream_printf("%s	%s\n", STR(keybuf), value);
+		    if (json_output == 0)
+			vstream_printf("%s	%s\n", STR(keybuf), value);
+		    else
+			vstream_printf("{\"%s\": \"%s\"}\n",
+			      quote_for_json(json_key_buf, STR(keybuf), -1),
+				   quote_for_json(json_val_buf, value, -1));
 		    found = 1;
 		    break;
 		}
@@ -794,7 +818,12 @@ static int postmap_query(const char *map_type, const char *map_name,
 	    msg_warn("table %s:%s should return NO RESULT in case of NOT FOUND",
 		     map_type, map_name);
 	}
-	vstream_printf("%s\n", value);
+	if (json_output == 0)
+	    vstream_printf("%s\n", value);
+	else
+	    vstream_printf("{\"%s\": \"%s\"}\n",
+			   quote_for_json(json_key_buf, key, -1),
+			   quote_for_json(json_val_buf, value, -1));
     }
     switch (dict->error) {
     case 0:
@@ -926,7 +955,12 @@ static void postmap_seq(const char *map_type, const char *map_name,
 	    }
 	    value = STR(unb64);
 	}
-	vstream_printf("%s	%s\n", key, value);
+	if (json_output == 0)
+	    vstream_printf("%s	%s\n", key, value);
+	else
+	    vstream_printf("{\"%s\": \"%s\"}\n",
+			   quote_for_json(json_key_buf, key, -1),
+			   quote_for_json(json_val_buf, value, -1));
     }
     if (dict->error)
 	msg_fatal("table %s:%s: sequence error: %m", dict->type, dict->name);
@@ -1007,7 +1041,7 @@ int     main(int argc, char **argv)
     /*
      * Parse JCL.
      */
-    while ((ch = GETOPT(argc, argv, "bc:d:fFhimnNopq:rsuUvw")) > 0) {
+    while ((ch = GETOPT(argc, argv, "bc:d:fFhijmnNopq:rsuUvw")) > 0) {
 	switch (ch) {
 	default:
 	    usage(argv[0]);
@@ -1043,6 +1077,13 @@ int     main(int argc, char **argv)
 	    update = 1;
 	    open_flags &= ~O_TRUNC;
 	    break;
+	case 'j':
+	    if (json_output == 0) {
+		json_output = 1;
+		json_key_buf = vstring_alloc(100);
+		json_val_buf = vstring_alloc(100);
+	    }
+	    break;
 	case 'm':
 	    postmap_flags |= POSTMAP_FLAG_MIME_KEY;
 	    break;
@@ -1085,6 +1126,8 @@ int     main(int argc, char **argv)
 	    break;
 	}
     }
+    if (json_output && !(sequence || query))
+	msg_fatal("option -j requires -q or -s");
     mail_conf_read();
     /* Enforce consistent operation of different Postfix parts. */
     import_env = mail_parm_split(VAR_IMPORT_ENVIRON, var_import_environ);
