@@ -552,7 +552,7 @@ static int smtp_get_effective_tls_level(DSN_BUF *why, SMTP_STATE *state)
 		       STR(iter->host));
 	    return (0);
 	}
-	break;
+ 	break;
     case SMTP_REQTLS_POLICY_ACT_OPP_TLS:
 	if (tls->level == TLS_LEV_NONE) {
 	    if (state->tls_stats)
@@ -1205,7 +1205,7 @@ static void smtp_connect_inet(SMTP_STATE *state, const char *nexthop,
 		/* XXX Assume there is no code at the end of this loop. */
 	    }
 	    /* Skip MX hosts that lack authorization. */
-	    if (!smtp_tls_authorize_mx_hostname(state->tls, addr->qname)) {
+	    if (!smtp_tls_authorize_mx_hostname(state->tls, SMTP_HNAME(addr))) {
 		continue;
 		/* XXX Assume there is no code at the end of this loop. */
 	    }
